#[macro_use]
extern crate rocket;
use rocket::form::Form;
use rocket::fs::{relative, FileServer};
use rocket_db_pools::{sqlx, Connection, Database};
use rocket_dyn_templates::{context, Template};
use sqlx::types::chrono;

#[derive(Database)]
#[database("dolltags")]
struct DollTags(sqlx::SqlitePool);

#[get("/")]
fn index() -> Template {
	Template::render("index", context! {})
}

#[get("/register")]
fn show_register() -> Template {
	Template::render("register", context! {})
}

// TODO: Validation
#[derive(Debug, FromForm)]
struct TagForm {
	pub ident: String,

	pub name: String,
	pub pronoun_subject: String,
	pub pronoun_object: String,
	pub pronoun_possessive: String,

	pub handler_name: String,
	pub handler_link: String,

	pub kind: String,
	pub breed: String,
	pub behaviour: String,
}

#[post("/register", data = "<tag>")]
fn handle_register(tag: Form<TagForm>) -> Template {
	println!("register: {:?}", tag);
	Template::render("register", context! {})
}

#[get("/profile?<ident>")]
async fn show_profile(mut db: Connection<DollTags>, ident: &str) -> Template {
	#[derive(Debug)]
	struct Tag {
		pub id: i64,
		pub created_at: chrono::NaiveDateTime,
	}
	sqlx::query_as!(
		Tag,
		r#"select id, created_at as "created_at!: chrono::NaiveDateTime" from doll_profiles"#
	)
	.fetch_one(&mut **db)
	.await
	.expect("fuck");

	Template::render(
		"show_profile",
		context! {
			ident,
		},
	)
}

#[launch]
fn rocket() -> _ {
	rocket::build()
		.attach(Template::fairing())
		.attach(DollTags::init())
		.mount("/assets", FileServer::from(relative!("/assets")))
		.mount(
			"/",
			routes![index, show_profile, show_register, handle_register],
		)
}
